--local CoolDownButtons = LibStub("AceAddon-3.0"):GetAddon("CoolDown Buttons")
CoolDownButtonsConfig = CoolDownButtons:NewModule("Config","AceConsole-3.0","AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("CoolDown Buttons", false)
local LSM = LibStub("LibSharedMedia-2.0")

local CDBCHelper = {}
local options = {}
local db

options.type = "group"
options.name = "CoolDown Buttons"
options.get = function( k ) return db[k.arg] end
options.set = function( k, v ) db[k.arg] = v; CoolDownButtonsConfig:UpdateConfig(); end
options.args = {}
options.plugins = {}

local opt_directions = {
	["up"]    = L["Up"],
	["down"]  = L["Down"],
	["left"]  = L["Left"],
	["right"] = L["Right"],
}
local opt_text_directions = {
	["up"]    = L["Above"],
	["down"]  = L["Below"],
	["left"]  = L["Left"],
	["right"] = L["Right"],
}

options.args.display = {
 	type = "group",
    name = L["Display Settings"],
	order = 0,
	args = {
        splitCooldowns = {
            order = 0,
            name = L["Split Cooldowns"],
            desc = L["Toggle showing Item and Spell Cooldowns as own rows or not."],
            type = "toggle",
            arg = "splitRows",
        },
        splitSoon = {
            order = 1,
            name = L["Split expiring Cooldown"],
            desc = L["Toggle showing Item and Spell Cooldowns in an additional row if they are expiring soon."],
            type = "toggle",
            arg = "splitSoon",
        },
        showTime = {
            order = 2,
            name = L["Show Time"],
            desc = L["Toggle showing Cooldown Time at the Buttons."],
            type = "toggle",
            arg = "showTime",
        },
        fontToUse = {
            order = 3,
            name = L["Font"],
            desc = "",
            type = "select",
            values = function() local fonts, newFonts = LSM:List("font"), {}; for k, v in pairs(fonts) do newFonts[v] = v; end return newFonts; end,
            get = function( k ) return db[k.arg] end,
            set = function( k, v ) db[k.arg] = v; CoolDownButtonsConfig:UpdateConfig(); end,
            arg = "font",
        },
        cooldownsSpells = {
            type = "group",
            name = L["Spell Cooldowns"],
            guiInline = true,
            order = 4,
            set = function( k, v ) db.anchors.spells[k.arg] = v; CoolDownButtonsConfig:UpdateConfig(); end,
            get = function( k ) return db.anchors.spells[k.arg] end,
            args = {
                showAnchor = {
                    order = 0,
                    name = L["Show Anchor"],
                    desc = L["Toggle showing Anchor."],
                    type = "toggle",
                    arg = "show",
                    width= "full",
                },
                displayDirection = {
                    order = 1,
                    name = L["Direction"],
                    desc = L["Direction from Anchor"],
                    type = "select",
                    values = opt_directions,
                    arg = "direction"
                },
                maxButtons = {
                    order = 2,
                    name = L["Max Buttons"],
                    desc = L["Maximal number of Buttons to display."],
                    type = "input",
                    arg = "maxbuttons",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.spells[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.spells[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 3,
                    type = "description",
                    name = "",
                },
                buttonScale = {
                    order = 4,
                    name = L["Button Scale"],
                    desc = L["Button scaling, this lets you enlarge or shrink your Buttons."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "scale",
                },
                buttonAlpha = {
                    order = 5,
                    name = L["Button Alpha"],
                    desc = L["Icon alpha value, this lets you change the transparency of the Button."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "alpha",
                },
                dummy2 = { -- Need another line break :)
                    order = 6,
                    type = "description",
                    name = "",
                },
                buttonPadding = {
                    order = 7,
                    name = L["Button Padding"],
                    desc = L["Space Between Buttons."],
                    type = "input",
                    arg = "buttonPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.spells[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.spells[k.arg]) end,
                },
                textPadding = {
                    order = 8,
                    name = L["Text Distance"],
                    desc = L["Distance of Text to Button."],
                    type = "input",
                    arg = "textPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.spells[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.spells[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 9,
                    type = "description",
                    name = "",
                },
                textSettings = {
                    order = 10,
                    name = L["Use Text Settings"],
                    desc = L["Toggle using extra Text Settings."],
                    type = "toggle",
                    arg = "textSettings",
                },
                textSide = {
                    disabled = function() return not db.anchors.spells.textSettings end,
                    order = 11,
                    name = L["Text Side"],
                    desc = L["Text Side from Button"],
                    type = "select",
                    values = opt_text_directions,
                    arg = "textSide"
                },
                dummy3 = { -- Need another line break :)
                    order = 12,
                    type = "description",
                    name = "",
                },
                textScale = {
                    disabled = function() return not db.anchors.spells.textSettings end,
                    order = 13,
                    name = L["Text Scale"],
                    desc = L["Text scaling, this lets you enlarge or shrink your Text."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "textScale",
                },
                textAlpha = {
                    disabled = function() return not db.anchors.spells.textSettings end,
                    order = 14,
                    name = L["Text Alpha"],
                    desc = L["Text alpha value, this lets you change the transparency of the Text."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "textAlpha",
                },
            },
        },
        cooldownsItems = {
            type = "group",
            name = L["Item Cooldowns"],
            guiInline = true,
            order = 5,
            hidden = function() return not db.splitRows end,
            set = function( k, v ) db.anchors.items[k.arg] = v; CoolDownButtonsConfig:UpdateConfig(); end,
            get = function( k ) return db.anchors.items[k.arg] end,
            args = {
                showAnchor = {
                    order = 0,
                    name = L["Show Anchor"],
                    desc = L["Toggle showing Anchor."],
                    type = "toggle",
                    arg = "show",
                    width= "full",
                },
                displayDirection = {
                    order = 1,
                    name = L["Direction"],
                    desc = L["Direction from Anchor"],
                    type = "select",
                    values = opt_directions,
                    arg = "direction"
                },
                maxButtons = {
                    order = 2,
                    name = L["Max Buttons"],
                    desc = L["Maximal number of Buttons to display."],
                    type = "input",
                    arg = "maxbuttons",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.items[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.items[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 3,
                    type = "description",
                    name = "",
                },
                buttonScale = {
                    order = 4,
                    name = L["Button Scale"],
                    desc = L["Button scaling, this lets you enlarge or shrink your Buttons."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "scale",
                },
                buttonAlpha = {
                    order = 5,
                    name = L["Button Alpha"],
                    desc = L["Icon alpha value, this lets you change the transparency of the Button."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "alpha",
                },
                dummy2 = { -- Need another line break :)
                    order = 6,
                    type = "description",
                    name = "",
                },
                buttonPadding = {
                    order = 7,
                    name = L["Button Padding"],
                    desc = L["Space Between Buttons."],
                    type = "input",
                    arg = "buttonPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.items[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.items[k.arg]) end,
                },
                textPadding = {
                    order = 8,
                    name = L["Text Distance"],
                    desc = L["Distance of Text to Button."],
                    type = "input",
                    arg = "textPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.items[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.items[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 9,
                    type = "description",
                    name = "",
                },
                textSettings = {
                    order = 10,
                    name = L["Use Text Settings"],
                    desc = L["Toggle using extra Text Settings."],
                    type = "toggle",
                    arg = "textSettings",
                },
                textSide = {
                    disabled = function() return not db.anchors.items.textSettings end,
                    order = 11,
                    name = L["Text Side"],
                    desc = L["Text Side from Button"],
                    type = "select",
                    values = opt_text_directions,
                    arg = "textSide"
                },
                dummy3 = { -- Need another line break :)
                    order = 12,
                    type = "description",
                    name = "",
                },
                textScale = {
                    disabled = function() return not db.anchors.items.textSettings end,
                    order = 13,
                    name = L["Text Scale"],
                    desc = L["Text scaling, this lets you enlarge or shrink your Text."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "textScale",
                },
                textAlpha = {
                    disabled = function() return not db.anchors.items.textSettings end,
                    order = 14,
                    name = L["Text Alpha"],
                    desc = L["Text alpha value, this lets you change the transparency of the Text."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "textAlpha",
                },
            },
        },
        cooldownsSoon = {
            type = "group",
            name = L["Expiring soon"],
            guiInline = true,
            order = 6,
            hidden = function() return not db.splitSoon end,
            set = function( k, v ) db.anchors.soon[k.arg] = v; CoolDownButtonsConfig:UpdateConfig(); end,
            get = function( k ) return db.anchors.soon[k.arg] end,
            args = {
                showAnchor = {
                    order = 0,
                    name = L["Show Anchor"],
                    desc = L["Toggle showing Anchor."],
                    type = "toggle",
                    arg = "show",
                },
                timeToSplit = {
                    order = 1,
                    name = L["Show X seconds before ready"],
                    desc = L["Sets the time in seconds when the Cooldown should switch to this bar."],
                    type = "input",
                    arg = "timeToSplit",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.soon[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.soon[k.arg]) end,
                },
                dummy0 = { -- Need another line break :)
                    order = 2,
                    type = "description",
                    name = "",
                },
                displayDirection = {
                    order = 3,
                    name = L["Direction"],
                    desc = L["Direction from Anchor"],
                    type = "select",
                    values = opt_directions,
                    arg = "direction"
                },
                maxButtons = {
                    order = 4,
                    name = L["Max Buttons"],
                    desc = L["Maximal number of Buttons to display."],
                    type = "input",
                    arg = "maxbuttons",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.soon[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.soon[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 5,
                    type = "description",
                    name = "",
                },
                buttonScale = {
                    order = 6,
                    name = L["Button Scale"],
                    desc = L["Button scaling, this lets you enlarge or shrink your Buttons."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "scale",
                },
                buttonAlpha = {
                    order = 7,
                    name = L["Button Alpha"],
                    desc = L["Icon alpha value, this lets you change the transparency of the Button."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "alpha",
                },
                dummy2 = { -- Need another line break :)
                    order = 8,
                    type = "description",
                    name = "",
                },
                buttonPadding = {
                    order = 9,
                    name = L["Button Padding"],
                    desc = L["Space Between Buttons."],
                    type = "input",
                    arg = "buttonPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.soon[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.soon[k.arg]) end,
                },
                textPadding = {
                    order = 10,
                    name = L["Text Distance"],
                    desc = L["Distance of Text to Button."],
                    type = "input",
                    arg = "textPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.soon[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.soon[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 11,
                    type = "description",
                    name = "",
                },
                textSettings = {
                    order = 12,
                    name = L["Use Text Settings"],
                    desc = L["Toggle using extra Text Settings."],
                    type = "toggle",
                    arg = "textSettings",
                },
                textSide = {
                    disabled = function() return not db.anchors.soon.textSettings end,
                    order = 13,
                    name = L["Text Side"],
                    desc = L["Text Side from Button"],
                    type = "select",
                    values = opt_text_directions,
                    arg = "textSide"
                },
                dummy3 = { -- Need another line break :)
                    order = 14,
                    type = "description",
                    name = "",
                },
                textScale = {
                    disabled = function() return not db.anchors.soon.textSettings end,
                    order = 15,
                    name = L["Text Scale"],
                    desc = L["Text scaling, this lets you enlarge or shrink your Text."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "textScale",
                },
                textAlpha = {
                    disabled = function() return not db.anchors.soon.textSettings end,
                    order = 16,
                    name = L["Text Alpha"],
                    desc = L["Text alpha value, this lets you change the transparency of the Text."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "textAlpha",
                },
            },
        },
        cooldownsSigle = {
            type = "group",
            name = L["Seperated Cooldowns"],
            guiInline = true,
            order = 7,
            set = function( k, v ) db.anchors.single[k.arg] = v; CoolDownButtonsConfig:UpdateConfig(); end,
            get = function( k ) return db.anchors.single[k.arg] end,
            args = {
                buttonScale = {
                    order = 4,
                    name = L["Button Scale"],
                    desc = L["Button scaling, this lets you enlarge or shrink your Buttons."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "scale",
                },
                buttonAlpha = {
                    order = 5,
                    name = L["Button Alpha"],
                    desc = L["Icon alpha value, this lets you change the transparency of the Button."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "alpha",
                },
                dummy2 = { -- Need another line break :)
                    order = 6,
                    type = "description",
                    name = "",
                },
                textPadding = {
                    order = 8,
                    name = L["Text Distance"],
                    desc = L["Distance of Text to Button."],
                    type = "input",
                    arg = "textPadding",
                    set = function( k, v ) if not (tonumber(v) == nil) then db.anchors.single[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                    get = function( k ) return tostring(db.anchors.single[k.arg]) end,
                },
                dummy1 = { -- Need another line break :)
                    order = 9,
                    type = "description",
                    name = "",
                },
                textSettings = {
                    order = 10,
                    name = L["Use Text Settings"],
                    desc = L["Toggle using extra Text Settings."],
                    type = "toggle",
                    arg = "textSettings",
                },
                textSide = {
                    disabled = function() return not db.anchors.single.textSettings end,
                    order = 11,
                    name = L["Text Side"],
                    desc = L["Text Side from Button"],
                    type = "select",
                    values = opt_text_directions,
                    arg = "textSide"
                },
                dummy3 = { -- Need another line break :)
                    order = 12,
                    type = "description",
                    name = "",
                },
                textScale = {
                    disabled = function() return not db.anchors.single.textSettings end,
                    order = 13,
                    name = L["Text Scale"],
                    desc = L["Text scaling, this lets you enlarge or shrink your Text."],
                    type = "range",
                    min = 0.5, max = 1.5, step = 0.05,
                    arg = "textScale",
                },
                textAlpha = {
                    disabled = function() return not db.anchors.single.textSettings end,
                    order = 14,
                    name = L["Text Alpha"],
                    desc = L["Text alpha value, this lets you change the transparency of the Text."],
                    type = "range",
                    min = 0.1, max = 1, step = 0.05,
                    arg = "textAlpha",
                },
            },
        },
    },
}

options.args.posting = {
	type = "group",
	name = L["Posting Settings"],
	order = 1,
	args = {
        enablePosting = {
            order = 0,
            name = L["Enable Chatpost"],
            desc = L["Toggle posting to Chat."],
            type = "toggle",
            arg = "chatPost",
        },
        chatPostList = {}, -- Later :)
        desc1 = {
            order = 2,
            type = "description",
            name = L["Note: Click on a Cooldown Button to post the remaining time to the above selectet Chats."],
        },
    }
}

options.args.posting.args.chatPostList = {
    type = "group",
    name = L["Post to:"],
    guiInline = true,
    order = 1,
    set = function( k, state ) db.posttochats[k.arg] = state; CoolDownButtonsConfig:UpdateConfig(); end,
    get = function( k ) return db.posttochats[k.arg] end,
    args = {
    	enablePostingSay = { arg = "say", order = 0, type = "toggle",
			name = L["Say"],
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingParty = { arg = "party", order = 1, type = "toggle",
			name = L["Party"],
			desc = L["Toggle posting to Chat."],
        }, dummy1 = { order = 2, type = "description", name = "", },
    	enablePostingRaid = { arg = "raid", order = 3, type = "toggle",
			name = L["Raid"],
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingGuild = { arg = "guild", order = 4, type = "toggle",
			name = L["Guild"],
			desc = L["Toggle posting to Chat."],
        }, dummy2 = { order = 5, type = "description", name = "", },
    	enablePostingOfficer = { arg = "officer", order = 6, type = "toggle",
			name = L["Officer"],
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingEmote = { arg = "emote", order = 7, type = "toggle",
			name = L["Emote"],
			desc = L["Toggle posting to Chat."],
        }, dummy3 = { order = 8, type = "description", name = "", },
    	enablePostingRaidWarn = { arg = "raidwarn", order = 9, type = "toggle",
			name = L["Raidwarning"],
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingBG = { arg = "battleground", order = 10, type = "toggle",
			name = L["Battleground"],
			desc = L["Toggle posting to Chat."],
        }, dummy6 = { order = 11, type = "description", name = "", },
    	enablePostingYell = { arg = "yell", order = 12, type = "toggle",
			name = L["Yell"],
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingChatFrame = { arg = "chatframe", order = 13, type = "toggle",
			name = L["Default Chatframe"],
			desc = L["Toggle posting to Chat."],
        },
        desc1 = {
            order = 14,
            type = "description",
            name = "Custom Channels:",
        },
    	enablePostingChannel5 = { arg = "channel5", order = 15, type = "toggle",
			name = "",
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingChannel6 = { arg = "channel6", order = 16, type = "toggle",
			name = "",
			desc = L["Toggle posting to Chat."],
        }, dummy7 = { order = 17, type = "description", name = "", },
    	enablePostingChannel7 = { arg = "channel7", order = 18, type = "toggle",
			name = "",
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingChannel8 = { arg = "channel8", order = 19, type = "toggle",
			name = "",
			desc = L["Toggle posting to Chat."],
        }, dummy8 = { order = 20, type = "description", name = "", },
    	enablePostingChannel9 = { arg = "channel9", order = 21, type = "toggle",
			name = "",
			desc = L["Toggle posting to Chat."],
        },
    	enablePostingChannel10 = { arg = "channel10", order = 22, type = "toggle",
			name = "",
			desc = L["Toggle posting to Chat."],
            disabled = true,
        },
    },
}

options.args.posting.args.messagesettings = {
    type = "group",
	name = L["Message Settings"],
	order = 0,
	args = {
        usedefault = {
            order = 0,
            name = L["Use default Message"],
            desc = L["Toggle posting the default Message."],
            type = "toggle",
            arg = "postdefaultmsg",
        },
        desc1 = {
            order = 1,
            type = "description",
            name = CoolDownButtons:gsub(L["The default message is: |cFFFFFFFF$RemainingCoolDown|r"],
                                          "$RemainingCoolDown", L["RemainingCoolDown"]).."\n\n",
        },
        desc2 = {
            order = 2,
            type = "description",
            name = CoolDownButtons:gsub(L["If \'|cFFFFFFFF$defaultmsg|r\' is disabled use the following Text"],
                                          "$defaultmsg", L["Use default Message"]),
        },
        custommessage = {
            order = 3,
            name = "",
            desc = L["Set the Text to post."],
            type = "input",
            arg  = "postcustom",
            width= "double",
        },
        desc3 = {
            order = 4,
            type = "description",
            name = L["Use |cFFFFFFFF$spell|r for spell name and |cFFFFFFFF$time|r for cooldowntime."],
        },
    },
}


options.args.savetopos = {
 	type = "group",
    name = L["Cooldown Settings"],
	order = 2,
	args = {
        maxSpellDuration = {
            order = 0,
            name = L["Max Spell Duration"],
            desc = L["Maximal Duration to show a Spell."],
            type = "input",
            arg = "maxSpellDuration",
            set = function( k, v ) if not (tonumber(v) == nil) then db[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
            get = function( k ) return tostring(db[k.arg]) end,
        },
        maxItemDuration = {
            order = 1,
            name = L["Max Item Duration"],
            desc = L["Maximal Duration to show a Item."],
            type = "input",
            arg = "maxItemDuration",
            set = function( k, v ) if not (tonumber(v) == nil) then db[k.arg] = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
            get = function( k ) return tostring(db[k.arg]) end,
        },
        desc3 = {
            order = 2,
            type = "description",
            name = "",
        },
    	showSpellAfterMax = {
            order = 3,
            type = "toggle",
			name = L["Show Spells later"],
			desc = L["Toggle Spells to display after remaining duration is below max duration."],
            arg = "spellShowAfterMaxDurationPassed",
        },
    	showItemsAfterMax = {
            order = 4,
            type = "toggle",
			name = L["Show Items later"],
			desc = L["Toggle Item to display after remaining duration is below max duration."],
            arg = "itemShowAfterMaxDurationPassed",
        },
        spells = {
            type = "group",
            name = L["Spell Positions"],
            order = 0,
            args = {
                desc = {
                    order = 0,
                    type = "description",
                    name = L["|cFFFFFFFFNote: The X and Y Axis are relative to your bottomleft screen cornor.|r"],
                },
            },
        },
        hidespells = {
            type = "group",
            name = L["Hide Spells"],
            order = 1,
            args = {
            },
        },
        items = {
            type = "group",
            name = L["Item Positions"],
            order = 2,
            args = {
                desc = {
                    order = 0,
                    type = "description",
                    name = L["|cFFFFFFFFNote: The X and Y Axis are relative to your bottomleft screen cornor.|r"],
                },
            },
        },
        hideitems = {
            type = "group",
            name = L["Hide Items"],
            order = 3,
            args = {
            },
        },
    },
}

local getProfilesOptionsTable
do
	local defaultProfiles
	--[[ Utility functions ]]
	-- get exisiting profiles + some default entries
	local tmpprofiles = {}
	local function getProfileList(db, common, nocurrent)
		-- clear old profile table
		local profiles = {}
		
		-- copy existing profiles into the table
		local curr = db:GetCurrentProfile()
		for i,v in pairs(db:GetProfiles(tmpprofiles)) do if not (nocurrent and v == curr) then profiles[v] = v end end
		
		-- add our default profiles to choose from
		for k,v in pairs(defaultProfiles) do
			if (common or profiles[k]) and not (k == curr and nocurrent) then
				profiles[k] = v
			end
		end
		return profiles
	end
	
	function getProfilesOptionsTable(db)
		defaultProfiles = {
			["Default"] = L["Default"],
			[db.keys.char] = L["Char:"] .. " " .. db.keys.char,
			[db.keys.realm] = L["Realm:"] .. " " .. db.keys.realm,
			[db.keys.class] = L["Class:"] .. " " .. UnitClass("player")
		}
		
		local tbl = {
			profiles = {
				type = "group",
				name = L["Profiles"],
				desc = L["Manage Profiles"],
				order = 20,
				args = {
					desc = {
						order = 1,
						type = "description",
						name = L["You can change the active profile of CoolDown Buttons, so you can have different settings for every character"] .. "\n",
					},
					descreset = {
						order = 9,
						type = "description",
						name = L["Reset the current profile back to its default values, in case your configuration is broken, or you simply want to start over."],
					},
					reset = {
						order = 10,
						type = "execute",
						name = L["Reset Profile"],
						desc = L["Reset the current profile to the default"],
						func = function() db:ResetProfile() CoolDownButtonsConfig:UpdateConfig(); end,
					},
					choosedesc = {
						order = 21,
						type = "description",
						name = "\n" .. L["You can create a new profile by entering a new name in the editbox, or choosing one of the already exisiting profiles."],
					},
					new = {
						name = L["New"],
						desc = L["Create a new empty profile."],
						type = "input",
						order = 30,
						get = function() return false end,
						set = function(info, value) db:SetProfile(value) end,
					},
					choose = {
						name = L["Current"],
						desc = L["Select one of your currently available profiles."],
						type = "select",
						order = 40,
						get = function() return db:GetCurrentProfile() end,
						set = function(info, value) db:SetProfile(value) CoolDownButtonsConfig:UpdateConfig(); end,
						values = function() return getProfileList(db, true) end,
					},
					deldesc = {
						order = 70,
						type = "description",
						name = "\n" .. L["Delete existing and unused profiles from the database"],
					},
					delete = {
						order = 80,
						type = "select",
						name = L["Delete a Profile"],
						desc = L["Deletes a profile from the database."],
						get = function() return false end,
						set = function(info, value) db:DeleteProfile(value) end,
						values = function() return getProfileList(db, nil, true) end,
						confirm = true,
						confirmText = L["Are you sure you want to delete the selected profile?"],
					},
				},
			},
		}
		return tbl
	end
end

function CoolDownButtonsConfig:InitPositions(state)

    options.args.savetopos.args.spells.args     = {}
    options.args.savetopos.args.hidespells.args = {}
    options.args.savetopos.args.items.args      = {}
    options.args.savetopos.args.hideitems.args  = {}

    local idx = 1
    for name, data in sortedpairs(db.saveToPos) do
        if type(data) == "table" then
            local arg = {
                type = "group",
                name = name,
                guiInline = true,
                order = idx,
                args = {
                    savethis = {
                        name  = CoolDownButtons:gsub(L["Save |cFFFFFFFF$obj|r to a consistent Position"], "$obj", name),
                        desc  = CoolDownButtons:gsub(L["Toggle saving of |cFFFFFFFF$obj|r."], "$obj", name),
                        arg   = name,
                        order = 0,
                        width = "full",
                        type  = "toggle",
                        set = function( k, state ) db.saveToPos[k.arg].saved = state; CoolDownButtonsConfig:UpdateConfig(); end,
                        get = function( k ) return db.saveToPos[k.arg].saved end,
                    },
                    xpos = {
                        order = 1,
                        name = L["X - Axis"],
                        desc = L["Set the Position on X-Axis."],
                        type = "input",
                        arg = name,
                        set = function( k, v ) if not (tonumber(v) == nil) then db.saveToPos[k.arg].pos.x = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                        get = function( k ) return tostring(db.saveToPos[k.arg].pos.x) end,
                    },
                    ypos = {
                        order = 2,
                        name = L["Y - Axis"],
                        desc = L["Set the Position on Y-Axis."],
                        type = "input",
                        arg = name,
                        set = function( k, v ) if not (tonumber(v) == nil) then db.saveToPos[k.arg].pos.y = tonumber(v); CoolDownButtonsConfig:UpdateConfig(); end end,
                        get = function( k ) return tostring(db.saveToPos[k.arg].pos.y) end,
                    }, dummy = { order = 3, type = "description", name = "", },
					move = {
						order = 4,
						type = "execute",
						name = L["Move"],
                        arg = name,
						func = function(k) CoolDownButtonsConfig:ShowFrameToMoveSavedCD(k.arg) end,
					},
					stopmove = {
						order = 5,
						type = "execute",
						name = L["Stop"],
						disabled = true,
                        arg = name,
						func = function(k) CoolDownButtonsConfig:HideFrameToMoveSavedCD(k.arg) CoolDownButtonsConfig:UpdateConfig(); end,
					},
                },
            }
            if data.cdtype == "spell" then
                options.args.savetopos.args.spells.args["obj"..idx] = arg
                options.args.savetopos.args.hidespells.args["obj"..idx] = {
                    order = idx,
                    type = "toggle",
                    name = CoolDownButtons:gsub(L["Show |cFFFFFFFF$obj|r"], "$obj", name),
                    desc = CoolDownButtons:gsub(L["Toggle to display |cFFFFFFFF$obj|r's CoolDown."], "$obj", name),
                    set = function( k, state ) db.saveToPos[k.arg].show = state; CoolDownButtonsConfig:UpdateConfig(); end,
                    get = function( k ) return db.saveToPos[k.arg].show end,
                    arg = name,
                }
            else
                options.args.savetopos.args.items.args["obj"..idx] = arg
                options.args.savetopos.args.hideitems.args["obj"..idx] = {
                    order = idx,
                    type = "toggle",
                    name = CoolDownButtons:gsub(L["Show |cFFFFFFFF$obj|r"], "$obj", name),
                    desc = CoolDownButtons:gsub(L["Toggle to display |cFFFFFFFF$obj|r's CoolDown."], "$obj", name),
                    set = function( k, state ) db.saveToPos[k.arg].show = state; CoolDownButtonsConfig:UpdateConfig(); end,
                    get = function( k ) return db.saveToPos[k.arg].show end,
                    arg = name,
                }
            end
            idx = idx + 1
        end
    end
    if state ~= "initial" then
        LibStub("AceConfigRegistry-3.0"):NotifyChange("CoolDown Buttons")
    end
end

function CoolDownButtonsConfig:OnInitialize()
	db = CoolDownButtons.db.profile

    self:InitPositions("initial")
    
    options.plugins["profiles"] = getProfilesOptionsTable(CoolDownButtons.db)
	self.options = options
    self:CHANNEL_UI_UPDATE() -- Force Update Channellist for the first time :)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("CoolDown Buttons", options)
	self:RegisterChatCommand("cdb", function() LibStub("AceConfigDialog-3.0"):Open("CoolDown Buttons") end)
	self:RegisterChatCommand("cooldownbuttons", function() LibStub("AceConfigDialog-3.0"):Open("CoolDown Buttons") end)
    self:RegisterMessage("CoolDownButtonsConfigChanged")
    self:RegisterEvent("CHANNEL_UI_UPDATE")
    
    self.moveableframe = nil
end

function CoolDownButtonsConfig:UpdateConfig()
	self:SendMessage("CoolDownButtonsConfigChanged")
end

function CoolDownButtonsConfig:ShowFrameToMoveSavedCD(name)
    if not self.moveableframe then
        self:createMovableFrame()
    end
	self.moveableframe:Show() 
    self.moveableframe:ClearAllPoints() 
	self.moveableframe:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.saveToPos[name].pos.x, db.saveToPos[name].pos.y) 
    local items = options.args.savetopos.args.items.args
    for k, v in pairs(items) do
        if k ~= "desc" then
            items[k].args.move.disabled = true
            if items[k].args.stopmove.arg == name then
                items[k].args.stopmove.disabled = false
            else
                items[k].args.stopmove.disabled = true
            end
        end
    end
    local spells = options.args.savetopos.args.spells.args
    for k, v in pairs(spells) do
        if k ~= "desc" then
            spells[k].args.move.disabled = true
            if spells[k].args.stopmove.arg == name then
                spells[k].args.stopmove.disabled = false
            else
                spells[k].args.stopmove.disabled = true
            end
        end
    end
end
function CoolDownButtonsConfig:HideFrameToMoveSavedCD(name)
	self.moveableframe:Hide()
    
    db.saveToPos[name].pos.x = tonumber(string.format("%.3f", self.moveableframe:GetLeft()))
    db.saveToPos[name].pos.y = tonumber(string.format("%.3f", self.moveableframe:GetBottom()))
    
    local items = options.args.savetopos.args.items.args
    for k, v in pairs(items) do
        if k ~= "desc" then
            items[k].args.move.disabled     = false
            items[k].args.stopmove.disabled = true
        end
    end
    local spells = options.args.savetopos.args.spells.args
    for k, v in pairs(spells) do
        if k ~= "desc" then
            spells[k].args.move.disabled     = false
            spells[k].args.stopmove.disabled = true
        end
    end
end

function CoolDownButtonsConfig:CoolDownButtonsConfigChanged()
	db = CoolDownButtons.db.profile
    CoolDownButtons:ResetCooldowns()
end

function CoolDownButtonsConfig:CHANNEL_UI_UPDATE()
    local chanlist = self.options.args.posting.args.chatPostList.args
    for i = 5, 10 do
        local channame = tostring(select(2,GetChannelName(i)))
        if not (channame == "nil") then
            chanlist["enablePostingChannel"..i].name = CoolDownButtons:gsub("(/"..i..") |cFFFFFFFF$defaultmsg|r", "$defaultmsg", channame )
            chanlist["enablePostingChannel"..i].disabled = false
        else 
            chanlist["enablePostingChannel"..i].name = "(/"..i..") No Channel"
            chanlist["enablePostingChannel"..i].disabled = true
            db.posttochats["channel"..i] = false -- Channel does not exist, so dont post here :P
        end
    end
    LibStub("AceConfigRegistry-3.0"):NotifyChange("CoolDown Buttons")
end

function CoolDownButtonsConfig:createMovableFrame()
    self.moveableframe = CoolDownButtons:createButton(-1, justMove)
    self.moveableframe:SetMovable(true);
    self.moveableframe:RegisterForDrag("LeftButton", "RightButton")
    self.moveableframe:RegisterForClicks("AnyDown")
    self.moveableframe:SetScript("OnDragStart", function(self) self:StartMoving() end)
    self.moveableframe:SetScript("OnDragStop",  function(self) self:StopMovingOrSizing(); end)
    self.moveableframe:Hide()
end




-- http://www.wowwiki.com/HOWTO:_Do_Tricks_With_Tables#Iterate_with_sorted_keys
function sortedpairs(t,comparator)
	local sortedKeys = {};
	table.foreach(t, function(k,v) table.insert(sortedKeys,k) end);
	table.sort(sortedKeys,comparator);
	local i = 0;
	local function _f(_s,_v)
		i = i + 1;
		local k = sortedKeys[i];
		if (k) then
			return k,t[k];
		end
	end
	return _f,nil,nil;
end
